library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity avalon_video is
  
  port (
    avs_s1_clk        : in  std_logic;
    avs_s1_reset_n    : in  std_logic;
    avs_s1_read       : in  std_logic;
    avs_s1_write      : in  std_logic;
    avs_s1_chipselect : in  std_logic;
    avs_s1_address    : in  std_logic_vector(1 downto 0);
    avs_s1_readdata   : out std_logic_vector(15 downto 0);
    avs_s1_writedata  : in  std_logic_vector(15 downto 0);
	
	reset_n	  : in std_logic;					  -- More reset!
	CLK_25	  : in std_logic;
    VGA_CLK   : out std_logic;                    -- Clock
    VGA_HS    : out std_logic;                    -- H_SYNC
    VGA_VS    : out std_logic;                    -- V_SYNC
    VGA_BLANK : out std_logic;                    -- BLANK
    VGA_SYNC  : out std_logic;                    -- SYNC
    VGA_R     : out std_logic_vector(9 downto 0); -- Red[9:0]
    VGA_G     : out std_logic_vector(9 downto 0); -- Green[9:0]
    VGA_B     : out std_logic_vector(9 downto 0)  -- Blue[9:0]
  );
    
end avalon_video;

architecture rtl of avalon_video is

signal clk : std_logic;

type ram_type is array(14 downto 0) of std_logic_vector(19 downto 0);

signal RAM : ram_type;
signal ram_address, display_address : std_logic_vector(3 downto 0);

constant HTOTAL       : integer := 800;
constant HSYNC        : integer := 96;
constant HBACK_PORCH  : integer := 48;
constant HACTIVE      : integer := 640;
constant HFRONT_PORCH : integer := 16;
  
constant VTOTAL       : integer := 525;
constant VSYNC        : integer := 2;
constant VBACK_PORCH  : integer := 33;
constant VACTIVE      : integer := 480;
constant VFRONT_PORCH : integer := 10;

--constant RECTANGLE_HSTART : integer := 100;
--constant RECTANGLE_HEND   : integer := 540;
--constant RECTANGLE_VSTART : integer := 100;
--constant RECTANGLE_VEND   : integer := 380;
-- Signals for the video controller
signal Hcount : std_logic_vector(9 downto 0);  -- Horizontal position (0-800)
signal Vcount : std_logic_vector(9 downto 0);  -- Vertical position (0-524)
signal EndOfLine, EndOfField : std_logic;

signal vga_hblank, vga_hsync,
 vga_vblank, vga_vsync : std_logic;  -- Sync. signals

signal visible_xpos: std_logic_vector(9 downto 0);
signal maze_x: std_logic_vector(4 downto 0);
signal maze_y: std_logic_vector(3 downto 0);
signal maze : std_logic;
signal rectangle_h, rectangle_v, rectangle : std_logic;  -- rectangle area
signal rectangle_hstart, rectangle_hend, rectangle_vstart, rectangle_vend : std_logic_vector(9 downto 0);

begin

  clk <= avs_s1_clk;
--  reset_n <= avs_s1_reset_n;
  ram_address <= avs_s1_address;
  maze_y <= (others => '0');
  visible_xpos <= Hcount - HSYNC - HBACK_PORCH;
  maze_x <= visible_xpos(9 downto 5);

  process (clk)
  begin
    if clk'event and clk = '1' then
	  RAM(0) <= "1111000011110000";
      if avs_s1_reset_n = '0' then
        avs_s1_readdata <= (others => '0');
        display_address <= (others => '0');        
--        counter <= (others => '0');        
--        counter_delay <= (others => '1');
      else
        if avs_s1_chipselect = '1' then
          if avs_s1_read = '1' then
--            avs_s1_readdata <= RAM(conv_integer(ram_address));
            avs_s1_readdata <= (others => '0');
--          elsif avs_s1_write = '1' then
--		;
--         RAM(conv_integer(ram_address)) <= avs_s1_writedata;
--			rectangle_hstart <= avs_s1_writedata(9 downto 0);
          end if;
        end if;
      end if;
    end if;
  end process;

	process (clk)
	begin
		if clk'event and clk = '1' then
			maze <= RAM(conv_integer(maze_y))(conv_integer(maze_x));
		end if;
	end process;
	
  HCounter : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      Hcount <= (others => '0');
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        Hcount <= (others => '0');
      else
        Hcount <= Hcount + 1;
      end if;      
    end if;
  end process HCounter;

  EndOfLine <= '1' when Hcount = HTOTAL - 1 else '0';
 
  VCounter: process (clk_25, reset_n)
  begin
    if reset_n = '1' then
      Vcount <= (others => '0');
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        if EndOfField = '1' then
          Vcount <= (others => '0');
        else
          Vcount <= Vcount + 1;
        end if;
      end if;
    end if;
  end process VCounter;

  EndOfField <= '1' when Vcount = VTOTAL - 1 else '0';

  -- State machines to generate HSYNC, VSYNC, HBLANK, and VBLANK

  HSyncGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      vga_hsync <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        vga_hsync <= '1';
      elsif Hcount = HSYNC - 1 then
        vga_hsync <= '0';
      end if;      
    end if;
  end process HSyncGen;
  
  HBlankGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      vga_hblank <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if Hcount = HSYNC + HBACK_PORCH then
        vga_hblank <= '0';
      elsif Hcount = HSYNC + HBACK_PORCH + HACTIVE then
        vga_hblank <= '1';
      end if;      
    end if;
  end process HBlankGen;

  VSyncGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      vga_vsync <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine ='1' then
        if EndOfField = '1' then
          vga_vsync <= '1';
        elsif Vcount = VSYNC - 1 then
          vga_vsync <= '0';
        end if;
      end if;      
    end if;
  end process VSyncGen;

  VBlankGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      vga_vblank <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        if Vcount = VSYNC + VBACK_PORCH - 1 then
          vga_vblank <= '0';
        elsif Vcount = VSYNC + VBACK_PORCH + VACTIVE - 1 then
          vga_vblank <= '1';
        end if;
      end if;      
    end if;
  end process VBlankGen;

  -- Rectangle generator

  RectangleHGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      rectangle_h <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if Hcount = HSYNC + HBACK_PORCH + unsigned(RECTANGLE_HSTART) then
        rectangle_h <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + unsigned(RECTANGLE_HSTART) + 16 then
        rectangle_h <= '0';
      end if;      
    end if;
  end process RectangleHGen;

  RectangleVGen : process (clk_25, reset_n)
  begin
    if reset_n = '1' then
      rectangle_v <= '0';
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        if Vcount = VSYNC + VBACK_PORCH - 1 + unsigned(RECTANGLE_VSTART) then
          rectangle_v <= '1';
        elsif Vcount = VSYNC + VBACK_PORCH - 1 + unsigned(RECTANGLE_VSTART) + 16 then
          rectangle_v <= '0';
        end if;
      end if;      
    end if;
  end process RectangleVGen;

  rectangle <= rectangle_h and rectangle_v;

  -- Registered video signals going to the video DAC

  VideoOut: process (clk_25, reset_n)
  begin
    if reset_n = '1' then
      VGA_R <= "0000000000";
      VGA_G <= "0000000000";
      VGA_B <= "0000000000";
    elsif clk_25'event and clk_25 = '1' then
      if rectangle = '1' then
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "1111111111";
      elsif vga_hblank = '0' and vga_vblank ='0' then
        VGA_R <= "1111111111";
        VGA_G <= "1111111111";
        VGA_B <= "1111111111";
      else
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000";    
      end if;
    end if;
  end process VideoOut;

  VGA_CLK <= clk_25;
  VGA_HS <= not vga_hsync;
  VGA_VS <= not vga_vsync;
  VGA_SYNC <= '0';
  VGA_BLANK <= not (vga_hsync or vga_vsync);
end rtl;
